/**
 *  '$RCSfile: DataFileServer.java,v $'
 *    Purpose: A Class that implements a socket server for data files that 
 *             writes the file to the servlet's local file system
 *  Copyright: 2000 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *    Authors: Chad Berkley
 *    Release: @release@
 *
 *   '$Author: berkley $'
 *     '$Date: 2001/01/18 23:15:20 $'
 * '$Revision: 1.8 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package edu.ucsb.nceas.metacat;

import java.net.*;
import java.io.*;
import java.util.Properties;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.sql.*;

public class DataFileServer extends DataFileUploadInterface
{
  MetaCatUtil util = new MetaCatUtil();
  static String filedir = "";

  protected Socket s;
  
  public DataFileServer(int port, String user, String sess_id)
  {
    super(port, user, sess_id);
    System.out.println("port: " + port + " user: " + user + " sess: " +
                       sess_id);
  }
  
  public void getFile(int port, String user, String sess_id)
  {
    filedir = util.getOption("datafilepath"); 
    ServerSocket server = null;
    Socket client = null;
    OutputStreamWriter osw = null;
    try
    {
      System.out.println("Starting on port " + port);
      System.out.println("Waiting for sess_id: " + sess_id);
      server = new ServerSocket((new Integer(port)).intValue());
      server.setSoTimeout(30000); //set a 30 second timeout
      client = server.accept();
      System.out.println("Accepted from " + client.getInetAddress());
    
      InputStream in = client.getInputStream();      //in from the client
      OutputStream sout = client.getOutputStream();  //out to the client
      osw = new OutputStreamWriter(sout);
      System.out.println("output stream received");
      // first read to get the file name
      byte[] str = new byte[1024];
      int val = -1;
      int i = 0;
      //get the filename
      while (val != 0) 
      {
        //System.out.println("i: " + i);
        val = in.read();
        str[i] = (byte)val;
        i++;
      }
      String filename = (new String(str,0, i-1)).trim();
      System.out.println("filename: " + filename);
      
      val = -1;
      i = 0;
      str = new byte[1024];
      //get the session id
      while (val != 0) 
      {
        val = in.read();
        str[i] = (byte)val;
        i++;
      }
      String session_id = (new String(str,0, i-1)).trim();
      
      //get the length of the file in bytes
      val = -1;
      i = 0;
      str = new byte[1024];
      while(val != 0)
      {
        val = in.read();
        str[i] = (byte)val;
        i++;
      }
      String stemp = (new String(str, 0, i-1)).trim();
      Long fsizetemp = new Long(stemp);
      long fsize = fsizetemp.longValue();
      //System.out.println("file size: " + fsize);
      
      if(!sess_id.equals(session_id))
      {//this is an unauthorized connection to this port
        System.out.println("unauthorized request");
        osw.write("<?xml version=\"1.0\"?><error>unauthorized request</error>");
        osw.flush();
        osw.close();
        return;
      }
      else
      {
        System.out.println("User authenticated on port " + port);
      }
      
      String restext=null;
      File outfile = new File(filedir + filename);
      System.out.println("outfile: " + filedir + filename);
      boolean nameInUse = outfile.exists();
      int filenametemp = 1;
      String fn = filename;
      while(nameInUse)
      {
        filename = fn + filenametemp;
        outfile = new File (filedir + filename);
        nameInUse = outfile.exists();
        filenametemp++;
      }
       
      try
      {
        AccessionNumber anum = new AccessionNumber();
        String accnum = anum.generate(null, "INSERT");
        
        FileOutputStream out = new FileOutputStream(outfile);
        byte[] buf = new byte[1024];
        int cnt = 0;
        int j = 0;
        while (j < fsize) 
        {
          cnt = in.read(buf, 0, 1024);
          if (cnt!=-1) 
          {
            out.write(buf, 0, cnt);
          }
          j += cnt;
        }

        out.flush();
        out.close();
        //put the new document info into the DB
        updateDB(accnum, filename, user);
        //System.out.println("sending new docid: " + accnum);
        osw.write("<?xml version=\"1.0\"?><docid>" + accnum + "</docid>");
        osw.flush();
        osw.close();
        //System.out.println("docid sent");
      }
      catch(Exception e)
      {
        System.out.println("error in DataFileServer.run(): " + e.getMessage());
        try
        {
          osw.write("<?xml version=\"1.0\"?><error>" + e.getMessage() + 
                    "</error>");
          osw.flush();
          osw.close();
        }
        catch(Exception ee)
        {
          System.out.println("error DataFileServer.run(): " + ee.getMessage());
        }
        e.printStackTrace(System.out);
      }
    } 
    catch(InterruptedIOException iioe)
    {
      //the accept timeout passed
      System.out.println("socket on port " + port + " timed out. " +
                         " (DataFileServer.run)");
    }
    catch (IOException ex) 
    {
      ex.printStackTrace();
      try
      {
        osw.write("<?xml version=\"1.0\"?><error>" + ex.getMessage() + 
                  "</error>");
        osw.flush();
        osw.close();
      }
      catch(Exception ee)
      {
        System.out.println("error in DataFileServer.run(): " + ee.getMessage());
      }
    } 
    finally 
    {
      try
      {
        server.close();
        //System.out.println("server socket closed");
        client.close();
        //System.out.println("client socket closed");
      } 
      catch (IOException ex) 
      {
        ex.printStackTrace();
      }
    }
  }
}
