/**
 *   '$Author: berkley $'
 *     '$Date: 2001/01/19 23:20:58 $'
 * '$Revision: 1.13 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
package edu.ucsb.nceas.metacat;

import java.sql.*;
import java.util.*;
import java.lang.Thread; 
import java.net.*;

public class RelationHandler implements Runnable
{
  private Thread btThread = null;
  private Connection conn = null;
  private DocumentImpl xmldoc = null;
  MetaCatUtil util = new MetaCatUtil();
  
  /** 
   * Constructor for this class.  finds all of the relations to a single xml
   * document and writes them to the database.  This includes transitive
   * relations.
   * @param xmldoc the xml document to index.
   */
  public RelationHandler(DocumentImpl xmldoc, Connection conn)
  {
    //this.conn = conn;
    try
    {
      conn = MetacatReplication.getDBConnection("RelationHandler." +
                                                "relationHandler");
    }
    catch(Exception e)
    {
      System.out.println("unable to get db connection in relationhandler." + 
                         "relationhandler: " + e.getMessage());
    }
    this.xmldoc = xmldoc;
    if(xmldoc.getDoctype().equals(util.getOption("packagedoctype")))
    { //make sure this doctype is a package document then run the thread
      btThread = new Thread(this);
      btThread.setPriority(Thread.MIN_PRIORITY);
      btThread.start();
    }
  }
  
  /**
   * The thread handler
   */ 
  public void run()
  {
    Connection dbconn = null;
    String docid = xmldoc.getDocID();
    // deletes all of the relations with a docid of @docid.
    deleteRelations(docid);
    //pseudo-code algorithm
    //for each new relation r in xml_nodes
    //  put r into xml_relation
    //  compare r to each relation already in xml_relation
    //  if r isrelatedto a row in xml_relation
    //    add a new row to xml_relation that represents this new relation
    try
    {
      dbconn = MetacatReplication.getDBConnection("RelationHandler." +
                                                  "run");
      PreparedStatement pstmt = dbconn.prepareStatement(
                                QuerySpecification.printPackageSQL(docid));
      pstmt.execute();
      
      //get the new relations out of xml_nodes
      ResultSet rs = pstmt.getResultSet();
      boolean hasmorerows = rs.next();
      while(hasmorerows)
      {
        String subject = rs.getString(1);
        String subjectDoctype = null;
        String paramDocid = null;
        URL subjectMurl = null;

        try
        {
          subjectMurl = new URL(subject);
          subjectDoctype = null;
          Hashtable murlParams = util.parseQuery(subjectMurl.getQuery());
          paramDocid = (String)murlParams.get("docid"); 
        }
        catch(MalformedURLException murle)
        { //assume this is just a docid not a url
          paramDocid = subject;
        }
        
        DocumentImpl subDoc = new DocumentImpl(dbconn, paramDocid);
        subjectDoctype = subDoc.getDoctype();
        String relationship = rs.getString(2);
        String object = rs.getString(3);
      
        //compare r to each relation in xml_relation
        
        pstmt = dbconn.prepareStatement("select subject, subdoctype, " +
                                      "relationship, object, objdoctype " +
                                      "from xml_relation");
        pstmt.execute();
        //get each relation in xml_relation for comparison
        ResultSet relations = pstmt.getResultSet();
        boolean hasmorerelations = relations.next();
        while(hasmorerelations)
        {
          String currentSub = relations.getString(1);
          String currentSubDoctype = relations.getString(2);
          String currentRelationship = relations.getString(3);
          String currentObj = relations.getString(4);
          String currentObjDoctype = relations.getString(5);
         
          if(object.equals(currentObj))
          {//there is a transitive relation so add a new relation to the table
            StringBuffer insertTransRelation = new StringBuffer();
            insertTransRelation.append("insert into xml_relation (docid, ");
            insertTransRelation.append(" subject, ");
            insertTransRelation.append("subdoctype, relationship, object, ");
            insertTransRelation.append("objdoctype) values ('");
            insertTransRelation.append(docid).append("', '");
            insertTransRelation.append(currentSub).append("', '");
            insertTransRelation.append(currentSubDoctype).append("', ");
            insertTransRelation.append("'hasTransitiveRelationTo', '");
            insertTransRelation.append(subject).append("', '");
            insertTransRelation.append(subjectDoctype).append("')");
            //System.out.println("sql1: " + insertTransRelation.toString());
            pstmt = dbconn.prepareStatement(insertTransRelation.toString());
            pstmt.execute(); 
            
            insertTransRelation = new StringBuffer();
            //put the same relation in with the subject and object switched
            insertTransRelation.append("insert into xml_relation (docid, ");
            insertTransRelation.append(" subject, ");
            insertTransRelation.append("subdoctype, relationship, object, ");
            insertTransRelation.append("objdoctype) values ('");
            insertTransRelation.append(docid).append("', '");
            insertTransRelation.append(subject).append("', '"); 
            insertTransRelation.append(subjectDoctype).append("', ");
            insertTransRelation.append("'hasTransitiveRelationTo', '");
            insertTransRelation.append(currentSub).append("', '");
            insertTransRelation.append(currentSubDoctype).append("')");
            //System.out.println("sql2: " + insertTransRelation.toString());
            pstmt = dbconn.prepareStatement(insertTransRelation.toString());
            pstmt.execute();
          }
          
          hasmorerelations = relations.next(); 
        }
        
        //get the current relations information
        String subDocid = null;
        String objDocid = null;
        String subDoctype = null;
        String objDoctype = null;
        
        try
        {
          URL subMurl = new URL(subject);
          if(subMurl.getQuery() != null)
          {
            Hashtable subMurlParams = util.parseQuery(subMurl.getQuery());
            subDocid = (String)subMurlParams.get("docid");
            if(subMurl.getProtocol().equals("metacat"))
            {
              subDoc = new DocumentImpl(dbconn, subDocid);
              subDoctype = subDoc.getDoctype();
            }
          }
          else
          {
            subDocid = subject;
          }
        }
        catch(MalformedURLException murle)
        { //assume this is just a docid not a url
          subDocid = subject;
        }
        
        try
        {
          URL objMurl = new URL(object); 
          if(objMurl.getQuery() != null)
          {
            Hashtable objMurlParams = util.parseQuery(objMurl.getQuery());
            objDocid = (String)objMurlParams.get("docid");
            if(objMurl.getProtocol().equals("metacat"))
            {
              DocumentImpl objDoc = new DocumentImpl(dbconn, objDocid);
              objDoctype = objDoc.getDoctype();
            }
          }
          else
          {
            objDocid = object;
          }
        }
        catch(MalformedURLException murle)
        { //assume this is just a docid
          objDocid = object;
        }
        
        
        //now that the comparisons are done, the new relation can be put
        //into xml_relation
        StringBuffer insertStmt = new StringBuffer();
        insertStmt.append("insert into xml_relation (docid, subject, ");
        insertStmt.append("subdoctype, ");
        insertStmt.append("relationship, object, objdoctype) values ('");
        insertStmt.append(docid).append("', '");
        insertStmt.append(subject).append("', '");
        insertStmt.append(subDoctype).append("', '");
        insertStmt.append(relationship).append("', '");
        insertStmt.append(object).append("', '");
        insertStmt.append(objDoctype).append("')");
        
        pstmt = dbconn.prepareStatement(insertStmt.toString());
        pstmt.execute(); 
        
        hasmorerows = rs.next();
      }
      
      dbconn.commit();
      pstmt.close();
      dbconn.close();
      btThread = null;

    } 
    catch(Exception e) 
    { 
      System.out.println("Error in relationHandler.run: " + e.getMessage());
      util.debugMessage("Error in relationHandler.run: " + e.getMessage());
      e.printStackTrace(System.out);
      btThread = null;
      try 
      { 
        conn.rollback();
      } 
      catch (SQLException sqle) {}
    }
  }
  
  /**
   * Deletes all of the relations with a docid of 'docid'.
   * @param docid the docid to delete.
   */
  public void deleteRelations(String docid)
  {
    Connection dbconn = null;
    try
    {
      dbconn = MetacatReplication.getDBConnection("RelationHandler."+
                                                  "deleteRelations");
    }
    catch(Exception ee)
    {
      System.out.println("error in relationHandler.deleteRelations: " +
                          ee.getMessage());
    }
    
    try
    {
      PreparedStatement pstmt = dbconn.prepareStatement("delete from " +
                             "xml_relation where docid like '" + docid + "'");
      pstmt.execute();
      pstmt.close();
      dbconn.close();
    }
    catch(Exception e)
    {
      System.out.println("error in RelationHandler.deleteRelations(): " + 
                          e.getMessage());
      e.printStackTrace(System.out);
      try 
      { 
        dbconn.rollback();
        dbconn.close();
      } 
      catch (SQLException sqle) {}
    }
    
  }
}
