/**
 *  '$RCSfile: marineServlet.java,v $'
 *    Purpose: class that implements the MARINE interface to the 
 *             metadata database.
 *  Copyright: 2000 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *    Authors: Chad Berkley
 * 
 *   '$Author: berkley $' 
 *     '$Date: 2000/09/05 15:45:34 $'
 * '$Revision: 1.10 $'
 */
 
package edu.ucsb.nceas.metacat.marine;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.*;
import java.sql.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import edu.ucsb.nceas.metacat.*;
import oracle.jdbc.driver.*;
import oracle.xml.parser.v2.*;
import oracle.xml.parser.v2.XMLParser;

public class marineServlet extends edu.ucsb.nceas.metacat.MetaCatServlet
{ 
  MetaCatUtil util = new MetaCatUtil();
  
  /**
   * Transorms an xml resultset document to html and sends it to the browser
   * @param resultdoc the string representation of the document that needs
   * to be transformed.
   * @param response the HttpServletResponse object bound to the client.
   * @param out the output stream to the client
   */
  protected void transformResultset(String resultdoc, 
                                  HttpServletResponse response,
                                  PrintWriter out)
  { 
    Connection conn = null;
    try
    {
      conn = util.getConnection();
      DBTransform trans = new DBTransform(conn);
      response.setContentType("text/html");
      trans.transformXMLDocument(resultdoc, "-//NCEAS//marineresultset//EN", 
                                 "-//W3C//HTML//EN", out);
      util.returnConnection(conn);
      util.closeConnections();
    }
    catch(Exception e)
    {
      out.println("error: " + e.getMessage());
      //if (conn != null) 
      {
        util.returnConnection(conn);
        util.closeConnections();
      }
    } 
  }
  
  /**
   * Formats the text in the <query> tag so that the tag name is the name
   * of the field and the content is the query itself. This is used to 
   * fill in the text boxes in the xsl created html form.
   * 
   * @param param is the query parameters from the CGI
   */
  protected String transformQuery(Hashtable params)
  {
    StringBuffer query = new StringBuffer();
    Object nextkey = null;
    Object nextelement = null;
    
    Enumeration elements = params.elements();
    Enumeration keys = params.keys();
    while(keys.hasMoreElements() && elements.hasMoreElements())
    {
      nextkey = keys.nextElement();
    	nextelement = elements.nextElement();
      //allow for more than value per field name
      for(int i=0; i<((String[])nextelement).length; i++)
      {
        if(((String[])nextelement).length > 1)
        { //this allows multiple values to be returned if there is more than
          //one of this field in the query.
          //(String)nextkey += String.valueOf(i);
        }
        query.append("<").append((String)nextkey).append(">");
        if(!((String[])nextelement)[i].equals(""))
        {
          query.append(((String[])nextelement)[i]);
        }
        query.append("</").append((String)nextkey).append(">");
      }
    }
    return query.toString();
  }
  
  /**
   * decodes the mouse click information coming from the client.
   * @param params the parameters from the CGI
   * @return action the action to be performed or "error" if an error was
   * generated
   */
  protected String decodeMouseAction(Hashtable params)
  {
    // Determine what type of request the user made
    // if the action parameter is set, use it as a default
    // but if the ypos param is set, calculate the action needed
    String action=null;
    long ypos = 0;
    try {
      ypos = (new Long(((String[])params.get("ypos"))[0]).longValue());
      //System.out.println("<P>YPOS IS " + ypos);
      if (ypos <= 13) {
        action = "getabstract";
      } else if (ypos > 13 && ypos <= 25) {
        action = "getdocument";
      } else if (ypos > 25) {
        action = "getdatadoc";
      }
      return action;
    } catch (Exception npe) {
      //
      // MBJ -- NOTE that this should be handled more gracefully with
      //        the new exception infrastructure -- this "error" return
      //        value is inappropriate
      //out.println("<P>Caught exception looking for Y value.");
      return "error";
    }  
  }
}

